/*
  Based on the ColorWheel SimbleeForMobile example by RF Digital Corp.
  Edits for WEAR Conference by SparkFun Electronics

  The sketch receives a Bluetooth Low Energy 4 connection,
  then uses PWM on a LilyPad Pixel to approximate the color
  selected.

  This sketch is used with SimbleeForMobile.  The sketch will maintain
  the color selected when disconnected, and can be easily
  modified when reconnected.
*/

/*
   Copyright (c) 2015 RF Digital Corp. All Rights Reserved.

   The source code contained in this file and all intellectual property embodied in
   or covering the source code is the property of RF Digital Corp. or its licensors.
   Your right to use this source code and intellectual property is non-transferable,
   non-sub licensable, revocable, and subject to terms and conditions of the
   SIMBLEE SOFTWARE LICENSE AGREEMENT.
   http://www.simblee.com/licenses/SimbleeSoftwareLicenseAgreement.txt

   THE SOURCE CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.

   This heading must NOT be removed from this file.
*/

#include <SimbleeForMobile.h>

/*-------------------------------------------------------------------
Variable declarations
---------------------------------------------------------------------*/
//Connect one of the LilyPad Pixel's DI tabs to pin 15
const int pixelPin = 15;

//Initialize color variables (start with white)
uint8_t red = 255;
uint8_t green = 255;
uint8_t blue = 255;

//UI element declarations
uint8_t rtextfield; //Red text field
uint8_t rslider; //Red slider

uint8_t gtextfield; //Green text field
uint8_t gslider; //Green slider

uint8_t btextfield; //Blue text field
uint8_t bslider; //Blue slider

uint8_t swatch; //Swatch rectangle to show selected color
uint8_t color_wheel; //Color wheel graphic


void setup() {
  //Set output pin for LilyPad Pixel
  pinMode(pixelPin, OUTPUT);

  //Only 15 characters available for .deviceName and .advertisementData TOTAL
  //Give your Simblee a unique name <15 characters
  SimbleeForMobile.deviceName = "NAME"; // Device name
  SimbleeForMobile.advertisementData = "Colors"; // Advert or Description

  // use a shared cache
  SimbleeForMobile.domain = "Simblee.com";

  SimbleeForMobile.begin();
  Serial.begin(9600); //Serial communication for sensors and debugging
}

void loop() {
  // Process must be called in the loop for SimbleeForMobile
  SimbleeForMobile.process();
}


void update();


/*-------------------------------------------------------------------
User Interface:
Here we'll determine the apperance and layout of the user interface
and assign objects that will update or send values
---------------------------------------------------------------------*/
void ui()
{
  //Setting a variable for dark gray color - to be used as app background
  color_t darkgray = rgb(0, 85, 85);

  SimbleeForMobile.beginScreen(WHITE);
  
   //Create a slider for the red channel of the Pixel. Use the slider to control 0-255 red values.
  SimbleeForMobile.drawText(25, 68, "R:", BLACK); // R Label
  rslider = SimbleeForMobile.drawSlider(55, 62, 175, 0, 255); // Red Slider
  rtextfield = SimbleeForMobile.drawTextField(245, 62, 50, 255, "", BLACK, WHITE); // Value text

  SimbleeForMobile.drawText(25, 105, "G:", BLACK); // G Label
  gslider = SimbleeForMobile.drawSlider(55, 99, 175, 0, 255); // Green Slider
  gtextfield = SimbleeForMobile.drawTextField(245, 99, 50, 255, "", BLACK, WHITE); // Value Text

  SimbleeForMobile.drawText(25, 142, "B:", BLACK); // B Label
  bslider = SimbleeForMobile.drawSlider(55, 136, 175, 0, 255); // Blue Slider
  btextfield = SimbleeForMobile.drawTextField(245, 136, 50, 255, "", BLACK, WHITE); // Value Text

  //Swatch Label
  SimbleeForMobile.drawText(25, 180, "Current Color", BLACK);
  //Swatch Rectangle Border
  SimbleeForMobile.drawRect(25, 200, 270, 40, WHITE);
  //Swatch Rectangle
  swatch = SimbleeForMobile.drawRect(26, 201, 268, 38, WHITE);

  //Draw the color wheel image
  color_wheel = SimbleeForMobile.drawImage(COLOR_WHEEL, 10, 250);

  SimbleeForMobile.setEvents(color_wheel, EVENT_COLOR);

  SimbleeForMobile.endScreen();

// Call update function (which we set to populate RGB values)
// (this must be done after endScreen() to force it to be done each time,
// otherwise the initial values will be populated from the cache)
  update();
}


/*-------------------------------------------------------------------
Update function:
Here we'll update the values of the sliders, the color of the swatch
in the app and send the color to the LilyPad Pixel on the Simblee
---------------------------------------------------------------------*/
void update()
{
  SimbleeForMobile.updateValue(rslider, red);
  SimbleeForMobile.updateValue(rtextfield, red);

  SimbleeForMobile.updateValue(gslider, green);
  SimbleeForMobile.updateValue(gtextfield, green);

  SimbleeForMobile.updateValue(bslider, blue);
  SimbleeForMobile.updateValue(btextfield, blue);

  SimbleeForMobile.updateColor(swatch, rgb(red, green, blue));

  //Update LilyPad Pixel color function
  RGB_Show(red, green, blue, pixelPin); 
}

/*-------------------------------------------------------------------
UI Events
---------------------------------------------------------------------*/
void ui_event(event_t &event)
{
  if (event.id == color_wheel)
  {
    red = event.red;
    green = event.green;
    blue = event.blue;
  }
  else if (event.id == rslider || event.id == rtextfield)
    red = event.value;
  else if (event.id == gslider || event.id == gtextfield)
    green = event.value;
  else if (event.id == bslider || event.id == btextfield)
    blue = event.value;
    
  update();
}


/*-------------------------------------------------------------------
This function was written for the LilyPad Pixel
Call the function and pass it R,G,B values and the pixel pin
---------------------------------------------------------------------*/
void RGB_Show(uint8_t r, uint8_t g, uint8_t b, uint32_t ledPin)
{
  uint8_t rgb[3] = {g, r, b};
  uint8_t *p = rgb;
  uint8_t *end = p + 3;

  while (p < end)
  {
    uint8_t pix = *p++;
    for (uint8_t mask = 0x80; mask; mask >>= 1)
    {
      if (pix & mask)
      {
        // T1H 760ns
        NRF_GPIO->OUTSET = (1UL << ledPin);
        NRF_GPIO->OUTSET = (1UL << ledPin);
        NRF_GPIO->OUTSET = (1UL << ledPin);

        // T1L 660ns
        NRF_GPIO->OUTCLR = (1UL << ledPin);
      }
      else
      {
        // T0H 380ns
        NRF_GPIO->OUTSET = (1UL << ledPin);

        // T0L 840ns
        NRF_GPIO->OUTCLR = (1UL << ledPin);
        NRF_GPIO->OUTCLR = (1UL << ledPin);
        NRF_GPIO->OUTCLR = (1UL << ledPin);
        NRF_GPIO->OUTCLR = (1UL << ledPin);
      }
    }
  }
  NRF_GPIO->OUTCLR = (1UL << ledPin);
}


